  IF ( narg == 0 ) THEN
     PRINT *,' usage : 
     PRINT *,'      '
     PRINT *,'     PURPOSE :'
     PRINT *,'        
     PRINT *,'      '
     PRINT *,'     ARGUMENTS :'
     PRINT *,'        
     PRINT *,'      '
     PRINT *,'     OPTIONS :'
     PRINT *,'        
     PRINT *,'      '
     PRINT *,'     REQUIRED FILES :'
     PRINT *,'        
     PRINT *,'      '
     PRINT *,'     OUTPUT : '
     PRINT *,'       netcdf file : ', TRIM(cf_out) 
     PRINT *,'         variables : ', TRIM(cv_out),' (    )'
     PRINT *,'      '
     PRINT *,'     SEE ALSO :'
     PRINT *,'       
     PRINT *,'      '
     STOP
  ENDIF
